@@ -1,5 +1,7 @@
 #![allow(non_snake_case)]
 #![macro_use]
+use std::collections::HashMap;
+
 pub mod event;
 pub mod event_stream;
 pub mod filter_chain;
@@ -244,6 +246,7 @@ define_filter!(
             Event::NoteOff(ev) => ev.port == self.0,
             Event::Ctrl(ev) => ev.port == self.0,
             Event::SysEx(ev) => ev.port == self.0,
+            #[cfg(feature = "osc")]
             Event::Osc(ev) => ev.port == self.0,
             _ => true,
         }
@@ -272,7 +275,6 @@ define_filter!(
     /// filter.run(&mut evs);
     /// assert_eq!(evs, vec![ev2, ev3]);
     /// ```
-
     PortsFilter(&'static [usize])
     fn filter_single(&self, ev: &Event) -> bool {
         match ev {
@@ -280,6 +282,7 @@ define_filter!(
             Event::NoteOff(ev) => self.0.contains(&ev.port),
             Event::Ctrl(ev) => self.0.contains(&ev.port),
             Event::SysEx(ev) => self.0.contains(&ev.port),
+            #[cfg(feature = "osc")]
             Event::Osc(ev) => self.0.contains(&ev.port),
             _ => true,
         }
@@ -570,7 +573,7 @@ define_generator!(
     /// # use rmididings::proc::*;
     /// let generator = NoteOn(60, 20);
     ///
-    /// let mut evs = EventStream::empty();
+    /// let mut evs = EventStream::none();
     /// generator.run(&mut evs);
     /// assert_eq!(evs, NoteOnEvent(0, 0, 60, 20))
     /// ```
@@ -594,7 +597,7 @@ define_generator!(
     /// # use rmididings::proc::*;
     /// let generator = NoteOff(65);
     ///
-    /// let mut evs = EventStream::empty();
+    /// let mut evs = EventStream::none();
     /// generator.run(&mut evs);
     /// assert_eq!(evs, NoteOffEvent(0, 0, 65))
     /// ```
@@ -618,7 +621,7 @@ define_generator!(
     /// # use rmididings::proc::*;
     /// let generator = Ctrl(7, 40);
     ///
-    /// let mut evs = EventStream::empty();
+    /// let mut evs = EventStream::none();
     /// generator.run(&mut evs);
     /// assert_eq!(evs, CtrlEvent(0, 0, 7, 40));
     /// ```
@@ -642,7 +645,7 @@ define_generator!(
     /// # use rmididings::proc::*;
     /// let generator = SysEx(&[0xf7, 0xf0]);
     ///
-    /// let mut evs = EventStream::empty();
+    /// let mut evs = EventStream::none();
     /// generator.run(&mut evs);
     /// assert_eq!(evs, SysExEvent(0, &[0xf7, 0xf0]));
     /// ```
@@ -680,6 +683,7 @@ define_modifier!(
             Event::NoteOff(ev) => ev.port = self.0,
             Event::Ctrl(ev) => ev.port = self.0,
             Event::SysEx(ev) => ev.port = self.0,
+            #[cfg(feature = "osc")]
             Event::Osc(ev) => ev.port = self.0,
             _ => {},
         }
@@ -936,101 +940,89 @@ impl FilterTrait for SceneSwitch {
     }
 }
 
-/// Change the current scene by the specified amount.
-///
-/// The argument is: _scene_delta_.
-///
-/// This event removes all non-scene-switch evens.
-///
-/// To go to the next scene, use `SceneSwitchOffset(1)`,
-/// to go to the previous scene, use `SceneSwitchOffset(-1)`.
-///
-/// Note that the scene is only switched when there are events, so
-/// that when an event filter discards all events, the scene switch
-/// is not done. It also means that you need to generate an event
-/// when putting this in a pre, init, exit or post patch.
-///
-/// # Examples
-///
-/// ```
-/// # use rmididings::proc::*;
-/// let generator = SceneSwitchOffset(1);
-///
-/// let mut evs = EventStream::none();
-/// generator.run(&mut evs);
-/// assert_eq!(evs, SceneSwitchOffsetEvent(1));
-/// ```
-pub struct SceneSwitchOffset(pub SceneOffset);
-impl FilterTrait for SceneSwitchOffset {
-    fn run(&self, evs: &mut EventStream) {
-        if evs.is_empty() { return; }
-        TypeFilter!(SceneSwitch).run(evs);
-        evs.push(SceneSwitchOffsetEvent(self.0));
+define_generator!(
+    /// Change the current scene by the specified amount.
+    ///
+    /// The argument is: _scene_delta_.
+    ///
+    /// To go to the next scene, use `SceneSwitchOffset(1)`,
+    /// to go to the previous scene, use `SceneSwitchOffset(-1)`.
+    ///
+    /// Note that the scene is only switched when there are events, so
+    /// that when an event filter discards all events, the scene switch
+    /// is not done. It also means that you need to generate an event
+    /// when putting this in a pre, init, exit or post patch.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use rmididings::proc::*;
+    /// let generator = SceneSwitchOffset(1);
+    ///
+    /// let mut evs = EventStream::none();
+    /// generator.run(&mut evs);
+    /// assert_eq!(evs, SceneSwitchOffsetEvent(1));
+    /// ```
+    SceneSwitchOffset(SceneOffset)
+    fn generate_single(&self) -> Event<'static> {
+        SceneSwitchOffsetEvent(self.0)
     }
-}
+);
 
-/// Switches to a specific subscene.
-///
-/// The argument is: _subscene_number_.
-///
-/// This event removes all non-scene-switch evens.
-///
-/// Note that the subscene is only switched when there are events, so
-/// that when an event filter discards all events, the subscene switch
-/// is not done. It also means that you need to generate an event
-/// when putting this in a pre, init, exit or post patch.
-///
-/// # Examples
-///
-/// ```
-/// # use rmididings::proc::*;
-/// let modifier = SubSceneSwitch(5);
-///
-/// let mut evs = EventStream::none();
-/// modifier.run(&mut evs);
-/// assert_eq!(evs, SubSceneSwitchEvent(5));
-/// ```
-pub struct SubSceneSwitch(pub SceneNum);
-impl FilterTrait for SubSceneSwitch {
-    fn run(&self, evs: &mut EventStream) {
-        if evs.is_empty() { return; }
-        TypeFilter!(SceneSwitch).run(evs);
-        evs.push(SubSceneSwitchEvent(self.0));
+define_generator!(
+    /// Switches to a specific subscene.
+    ///
+    /// The argument is: _subscene_number_.
+    ///
+    /// Note that the subscene is only switched when there are events, so
+    /// that when an event filter discards all events, the subscene switch
+    /// is not done. It also means that you need to generate an event
+    /// when putting this in a pre, init, exit or post patch.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use rmididings::proc::*;
+    /// let modifier = SubSceneSwitch(5);
+    ///
+    /// let mut evs = EventStream::none();
+    /// modifier.run(&mut evs);
+    /// assert_eq!(evs, SubSceneSwitchEvent(5));
+    /// ```
+    SubSceneSwitch(SceneNum)
+    fn generate_single(&self) -> Event<'static> {
+        SubSceneSwitchEvent(self.0)
     }
-}
+);
 
-/// Change the current subscene by the specified amount.
-///
-/// The argument is: _subscene_delta_.
-///
-/// This event removes all non-scene-switch evens.
-///
-/// To go to the next scene, use `SubSceneSwitchOffset(1)`,
-/// to go to the previous scene, use `SubSceneSwitchOffset(-1)`.
-///
-/// Note that the subscene is only switched when there are events, so
-/// that when an event filter discards all events, the subscene switch
-/// is not done. It also means that you need to generate an event
-/// when putting this in a pre, init, exit or post patch.
-///
-/// # Examples
-///
-/// ```
-/// # use rmididings::proc::*;
-/// let generator = SubSceneSwitchOffset(1);
-///
-/// let mut evs = EventStream::none();
-/// generator.run(&mut evs);
-/// assert_eq!(evs, SubSceneSwitchOffsetEvent(1));
-/// ```
-pub struct SubSceneSwitchOffset(pub SceneOffset);
-impl FilterTrait for SubSceneSwitchOffset {
-    fn run(&self, evs: &mut EventStream) {
-        if evs.is_empty() { return; }
-        TypeFilter!(SceneSwitch).run(evs);
-        evs.push(SubSceneSwitchOffsetEvent(self.0));
+define_generator!(
+    /// Change the current subscene by the specified amount.
+    ///
+    /// The argument is: _subscene_delta_.
+    ///
+    /// To go to the next scene, use `SubSceneSwitchOffset(1)`,
+    /// to go to the previous scene, use `SubSceneSwitchOffset(-1)`.
+    ///
+    /// Note that the subscene is only switched when there are events, so
+    /// that when an event filter discards all events, the subscene switch
+    /// is not done. It also means that you need to generate an event
+    /// when putting this in a pre, init, exit or post patch.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use rmididings::proc::*;
+    /// let generator = SubSceneSwitchOffset(1);
+    ///
+    /// let mut evs = EventStream::none();
+    /// generator.run(&mut evs);
+    /// assert_eq!(evs, SubSceneSwitchOffsetEvent(1));
+    /// ```
+    SubSceneSwitchOffset(SceneOffset)
+    fn generate_single(&self) -> Event<'static> {
+        SubSceneSwitchOffsetEvent(self.0)
     }
-}
+);
 
 #[doc(hidden)]
 pub struct _Init<'a>(pub Box<dyn FilterTrait + 'a>);
@@ -1266,24 +1258,62 @@ macro_rules! Not {
     };
 }
 
+#[doc(hidden)]
+pub struct _Process(pub Box<dyn Fn(&Event) -> Box<dyn FilterTrait>>);
+#[doc(hidden)]
+impl FilterTrait for _Process {
+    fn run(&self, evs: &mut EventStream) {
+       let mut results: HashMap<usize, EventStream> = HashMap::new();
+
+        // First gather all resulting EventStreams from the function invocations.
+        for (i, ev) in evs.iter().enumerate() {
+            let mut evs = EventStream::from(ev);
+            self.0(ev).run(&mut evs);
+            results.insert(i, evs);
+       }
+
+        // Then replace the events by their results.
+        for (i, r_evs) in results {
+            evs.splice(i..i+1, r_evs);
+        }
+
+        evs.dedup();
+    }
+}
+
 /// Process the incoming event using a custom function, returning a patch.
 ///
 /// Any other processing will be stalled until function returns, so this should only be used with
 /// functions that don’t block.
-// pub struct Process<'a>(dyn Fn(&Event) -> FilterChain<'a>);
-// impl FilterTrait for Process<'_> {
-//     fn run(&self, evs: &mut EventStream) {
-//         let filters: Vec<Box<dyn FilterTrait>> = vec![];
+///
+/// # Examples
+///
+/// ```
+/// # #[macro_use] extern crate rmididings;
+/// # use rmididings::proc::*;
+///
+/// # fn main() {
+/// let filter = Process!(|ev: &Event| -> Box<dyn FilterTrait> {
+///     match ev {
+///         Event::NoteOff(ev) => Box::new(NoteOn(ev.note + 1, 40)),
+///         _ => Box::new(Pass()),
+///     }
+/// });
+///
+/// let mut evs = EventStream::from(NoteOffEvent(0,0,60));
+/// filter.run(&mut evs);
+/// assert_eq!(evs, NoteOnEvent(0,0,61,40));
+/// # }
+/// ```
+#[macro_export]
+macro_rules! Process {
+    ( $f:expr ) => { _Process(Box::new($f)) };
+}
 
-//         for ev in evs.iter() {
-//             filters.push(Box::new(self.0(&ev)));
-//         }
-
-//         for f in filters {
-//             f.run(evs);
-//         }
-//     }
-// }
+#[macro_export]
+macro_rules! ProcessCtrl {
+    ( $f:expr ) => { _Process(Box::new($f)) };
+}
 
 #[cfg(feature = "osc")]
 pub mod osc;
